@@ -26,8 +26,8 @@ __copyright__ = """
 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 """
 
-__version__ = "0.7.1"
+__version__ = "0.8.1"
 
 repo_root = pathlib.Path(__file__).parent
 install_requires = (repo_root / "requirements.txt").read_text(encoding="utf-8").split('\n')
-extras_require = {"all": []}
+extras_require = {"terminals": ["psutil>=5.8.0"], "all": ["psutil>=5.8.0"]}
