@@ -60,14 +60,15 @@ Command line options.
 
 # stdlib
 import inspect
-from typing import Any, Callable, List, Optional, cast
+from typing import Any, Callable, Iterable, List, Optional, Tuple, TypeVar, cast
 
 # 3rd party
 import click
-from click import Context, Option, OptionParser
+from click import Argument, Context, IntRange, Option, OptionParser
 from click.decorators import _param_memo  # type: ignore
 
 # this package
+import consolekit.input
 from consolekit._types import Callback, _ConvertibleType
 
 __all__ = [
@@ -79,10 +80,32 @@ __all__ = [
 		"MultiValueOption",
 		"flag_option",
 		"auto_default_option",
+		"auto_default_argument",
 		]
 
+_A = TypeVar("_A", bound=click.Argument)
+_C = TypeVar("_C", bound=click.Command)
 
-def verbose_option(help_text: str = "Show verbose output.") -> Callable:
+
+class VerboseVersionCountType(click.IntRange):
+	"""
+	Subclass of :class:`click.IntRange` which doesn't show the range of valid values.
+
+	.. versionadded:: 0.8.1
+	"""
+
+	def __init__(self):
+		super().__init__(min=0)
+
+	def _describe_range(self):
+		"""
+		Describe the range for use in help text.
+		"""
+
+		return ''
+
+
+def verbose_option(help_text: str = "Show verbose output.") -> Callable[..., click.Command]:
 	"""
 	Adds an option (via the parameter ``verbose``: :class:`int`) to enable verbose output.
 
@@ -93,23 +116,35 @@ def verbose_option(help_text: str = "Show verbose output.") -> Callable:
 	:param help_text: The help text for the option.
 	"""
 
-	return click.option(
-			"-v",
-			"--verbose",
-			count=True,
-			help=help_text,
-			)
+	return click.option("-v", "--verbose", count=True, help=help_text, type=VerboseVersionCountType())
 
 
-def version_option(callback: Callable[[Context, Option, int], Any]) -> Callable:
+def version_option(callback: Callable[[Context, Option, int], Any]) -> Callable[..., click.Command]:
 	"""
 	Adds an option to show the version and exit.
 
 	The option can be provided multiple times by the user.
+	The count is stored as an integer and passed as the third parameter to the callback function.
 
 	.. versionadded:: 0.4.0
 
 	:param callback: The callback to invoke when the option is provided.
+
+	The callback function might look like:
+
+	.. code-block:: python
+
+		def version_callback(ctx: click.Context, param: click.Option, value: int):
+			if not value or ctx.resilient_parsing:
+				return
+
+			if value > 1:
+				click.echo(f"consolekit version {__version__}, Python {sys.version}")
+			else:
+				click.echo(f"consolekit version {__version__}")
+
+			ctx.exit()
+
 	"""
 
 	return click.option(
@@ -118,11 +153,12 @@ def version_option(callback: Callable[[Context, Option, int], Any]) -> Callable:
 			expose_value=False,
 			is_eager=True,
 			help="Show the version and exit.",
+			type=VerboseVersionCountType(),
 			callback=cast(Callback, callback),
 			)
 
 
-def colour_option(help_text="Whether to use coloured output.") -> Callable:
+def colour_option(help_text="Whether to use coloured output.") -> Callable[..., click.Command]:
 	"""
 	Adds an option (via the parameter ``colour``: :class:`bool`) to enable verbose output.
 
@@ -138,10 +174,12 @@ def colour_option(help_text="Whether to use coloured output.") -> Callable:
 			)
 
 
-def force_option(help_text: str) -> Callable:
+def force_option(help_text: str) -> Callable[..., click.Command]:
 	"""
 	Decorator to add the ``-f / --force`` option to a click command.
 
+	The value is exposed via the parameter ``force``: :class:`bool`.
+
 	.. versionadded:: 0.5.0
 
 	:param help_text: The help text for the option.
@@ -150,10 +188,12 @@ def force_option(help_text: str) -> Callable:
 	return flag_option("-f", "--force", help=help_text)
 
 
-def no_pager_option(help_text="Disable the output pager.") -> Callable:
+def no_pager_option(help_text="Disable the output pager.") -> Callable[..., click.Command]:
 	"""
 	Decorator to add the ``--no-pager`` option to a click command.
 
+	The value is exposed via the parameter ``no_pager``: :class:`bool`.
+
 	.. versionadded:: 0.5.0
 
 	:param help_text: The help text for the option.
@@ -162,7 +202,7 @@ def no_pager_option(help_text="Disable the output pager.") -> Callable:
 	return flag_option("--no-pager", help=help_text)
 
 
-def flag_option(*args, default: Optional[bool] = False, **kwargs) -> Callable:
+def flag_option(*args, default: Optional[bool] = False, **kwargs) -> Callable[..., click.Command]:
 	r"""
 	Decorator to a flag option to a click command.
 
@@ -181,7 +221,7 @@ def flag_option(*args, default: Optional[bool] = False, **kwargs) -> Callable:
 			)
 
 
-def auto_default_option(*param_decls, **attrs) -> Callable:
+def auto_default_option(*param_decls, **attrs) -> Callable[..., click.Command]:
 	"""
 	Attaches an option to the command, with a default value determined from the decorated function's signature.
 
@@ -195,7 +235,7 @@ def auto_default_option(*param_decls, **attrs) -> Callable:
 	:param cls: the option class to instantiate. This defaults to :class:`click.Option`.
 	"""
 
-	def decorator(f):
+	def decorator(f: _C) -> _C:
 		option_attrs = attrs.copy()
 
 		if "help" in option_attrs:
@@ -206,12 +246,44 @@ def auto_default_option(*param_decls, **attrs) -> Callable:
 		option = OptionClass(param_decls, **option_attrs)
 		_param_memo(f, option)
 
-		signature: inspect.Signature = inspect.signature(f.callback)
+		_get_default_from_callback_and_set(f, option)
 
-		param_default = signature.parameters[option.name].default
+		return f
+
+	return decorator
+
+
+def _get_default_from_callback_and_set(command: click.Command, param: click.Parameter):
+	if command.callback is not None:
+		# The callback *can* be None, for a no-op
+		signature: inspect.Signature = inspect.signature(command.callback)
+
+		param_default = signature.parameters[param.name].default
 
 		if param_default is not inspect.Signature.empty:
-			option.default = param_default
+			param.default = param_default
+
+
+def auto_default_argument(*param_decls, **attrs) -> Callable[..., click.Argument]:
+	"""
+	Attaches an argument to the command, with a default value determined from the decorated function's signature.
+
+	All positional arguments are passed as parameter declarations to :class:`click.Argument`;
+	all keyword arguments are forwarded unchanged (except ``cls``).
+	This is equivalent to creating an :class:`click.Argument` instance manually
+	and attaching it to the :attr:`click.Argument.params` list.
+
+	.. versionadded:: 0.8.0
+
+	:param cls: the option class to instantiate. This defaults to :class:`click.Argument`.
+	"""
+
+	def decorator(f: _C) -> _C:
+		ArgumentClass = attrs.pop("cls", Argument)
+		argument = ArgumentClass(param_decls, **attrs)
+		_param_memo(f, argument)
+
+		_get_default_from_callback_and_set(f, argument)
 
 		return f
 
@@ -234,26 +306,41 @@ class MultiValueOption(click.Option):
 		The later is converted into the former automatically if supported.
 	:param required: Controls whether this is optional.
 	:param default: The default value if omitted.
-		This can also be a callable, in which case it's invoked when the default is needed without any arguments.
+		This can also be a callable, in which case it is invoked when the default is needed without any arguments.
 	:param callback: A callback that should be executed after the parameter was matched.
 		This is called as ``fn(ctx, param, value)`` and needs to return the value.
 	:param metavar: How the value is represented in the help page.
 	:param expose_value: If :py:obj:`True` then the value is passed onwards to the command callback
-		and stored on the context, otherwise it's skipped.
+		and stored on the context, otherwise it is skipped.
 	:param is_eager: Eager values are processed before non eager ones.
 
 	.. versionadded:: 0.6.0
+
+	Example usage:
+
+	.. code-block:: python
+
+		@click.option(
+				"--select",
+				type=click.STRING,
+				help="The checks to enable",
+				cls=MultiValueOption,
+				)
+		@click_command()
+		def main(select: Iterable[str]):
+			select = list(select)
+
 	"""
 
 	def __init__(
 			self,
 			param_decls: Optional[List[str]] = None,
 			show_default: bool = False,
-			help: Optional[str] = None,  # noqa: A002
+			help: Optional[str] = None,  # noqa: A002  # pylint: disable=redefined-builtin
 			hidden: bool = False,
-			type: Optional[_ConvertibleType] = None,  # noqa: A002
+			type: Optional[_ConvertibleType] = None,  # noqa: A002  # pylint: disable=redefined-builtin
 			required: bool = False,
-			default: Optional[Any] = None,
+			default: Optional[Any] = (),
 			callback: Optional[Callback] = None,
 			metavar: Optional[str] = None,
 			expose_value: bool = True,
@@ -312,3 +399,55 @@ class MultiValueOption(click.Option):
 				break
 
 		return retval
+
+	def process_value(self, ctx: click.Context, value: Any) -> Optional[Tuple]:
+		"""
+		Given a value and context, converts the value as necessary.
+
+		:param ctx:
+		:param value:
+		"""
+
+		# If the value we were given is None we do nothing.
+		# This way code that calls this can easily figure out if something was not provided.
+		# Otherwise, it would be converted into an empty tuple for multiple invocations,
+		# which is inconvenient.
+
+		# assert isinstance(value, tuple), type(value)
+
+		if value is not None:
+			if isinstance(value, Iterable) and not isinstance(value, str):
+				return tuple(self.type_cast_value(ctx, v) for v in value)
+			elif value in ("()", str(self.default)) or not value:
+				return self.default
+			else:
+				return self.type_cast_value(ctx, value)
+		else:
+			return None
+
+
+class _Option(click.Option):
+
+	def prompt_for_value(self, ctx):
+		"""
+		This is an alternative flow that can be activated in the full value processing if a value does not exist.
+
+		It will prompt the user until a valid value exists and then returns the processed value as result.
+		"""
+
+		# Calculate the default before prompting anything to be stable.
+		default = self.get_default(ctx)
+
+		# If this is a prompt for a flag we need to handle this differently.
+		if self.is_bool_flag:
+			return consolekit.input.confirm(self.prompt, default)
+
+		return consolekit.input.prompt(
+				self.prompt,
+				default=default,
+				type=self.type,
+				hide_input=self.hide_input,
+				show_choices=self.show_choices,
+				confirmation_prompt=self.confirmation_prompt,
+				value_proc=lambda x: self.process_value(ctx, x),
+				)
